#include "binary_arithmetic_filter.hpp"
#include "workflow_graph.hpp"

namespace xios
{
  CScalarFieldArithmeticFilter::CScalarFieldArithmeticFilter(CGarbageCollector& gc, const std::string& op, double value)
    : CFilter(gc, 1, this)
    , op(operatorExpr.getOpScalarField(op))
    , value(value)
  { 
    StdString input_op_expression=op;
    if(input_op_expression == "add")
      op_expression = "+";
    else if(input_op_expression == "minus")
      op_expression = "-";
    else if(input_op_expression == "mult")
      op_expression = "x";
    else if(input_op_expression == "div")
      op_expression = "/";
    else if(input_op_expression == "eq")
      op_expression = "=";
    else if(input_op_expression == "lt")
      op_expression = "<";
    else if(input_op_expression == "gt")
      op_expression = ">";
    else if(input_op_expression == "le")
      op_expression = "<=";
    else if(input_op_expression == "ge")
      op_expression = ">=";
    else if(input_op_expression == "ne")
      op_expression = "!=";
    else
      op_expression = " ";
  };

  CDataPacketPtr CScalarFieldArithmeticFilter::apply(std::vector<CDataPacketPtr> data)
  {
    if(this->tag)
    {
      this->filterID = InvalidableObject::filterIdGenerator++;    

      if(CWorkflowGraph::mapFieldToFilters_ptr==0) CWorkflowGraph::mapFieldToFilters_ptr = new std::unordered_map <StdString, vector <int> >;
      if(CWorkflowGraph::mapFilters_ptr==0) CWorkflowGraph::mapFilters_ptr = new std::unordered_map <int, StdString>;

      (*CWorkflowGraph::mapFilters_ptr)[this->filterID] = "S "+op_expression +" F Filter" ;
      std::cout<<"CScalarFieldArithmeticFilter::apply filter tag = "<<this->tag<<std::endl;

      StdString str = data[0]->fieldID +" ts=" + to_string(data[0]->timestamp);
      (*CWorkflowGraph::mapFieldToFilters_ptr)[str].push_back(data[0]->src_filterID);
      (*CWorkflowGraph::mapFieldToFilters_ptr)[str].push_back(this->filterID);
    }

    CDataPacketPtr packet(new CDataPacket);
    packet->date = data[0]->date;
    packet->timestamp = data[0]->timestamp;
    packet->status = data[0]->status;
    if(this->tag) packet->src_filterID = this->filterID;
    packet->fieldID = this->output_field_id;

    if (packet->status == CDataPacket::NO_ERROR)
      packet->data.reference(op(value, data[0]->data));

    return packet;
  }

  CFieldScalarArithmeticFilter::CFieldScalarArithmeticFilter(CGarbageCollector& gc, const std::string& op, double value)
    : CFilter(gc, 1, this)
    , op(operatorExpr.getOpFieldScalar(op))
    , value(value)
  { 
    StdString input_op_expression=op;
    if(input_op_expression == "add")
      op_expression = "+";
    else if(input_op_expression == "minus")
      op_expression = "-";
    else if(input_op_expression == "mult")
      op_expression = "x";
    else if(input_op_expression == "div")
      op_expression = "/";
    else if(input_op_expression == "pow")
      op_expression = "^";
    else if(input_op_expression == "eq")
      op_expression = "=";
    else if(input_op_expression == "lt")
      op_expression = "<";
    else if(input_op_expression == "gt")
      op_expression = ">";
    else if(input_op_expression == "le")
      op_expression = "<=";
    else if(input_op_expression == "ge")
      op_expression = ">=";
    else if(input_op_expression == "ne")
      op_expression = "!=";
    else
      op_expression = " ";
  };

  CDataPacketPtr CFieldScalarArithmeticFilter::apply(std::vector<CDataPacketPtr> data)
  {
    if(this->tag)
    {
      this->filterID = InvalidableObject::filterIdGenerator++;
 
      if(CWorkflowGraph::mapFieldToFilters_ptr==0) CWorkflowGraph::mapFieldToFilters_ptr = new std::unordered_map <StdString, vector <int> >;
      if(CWorkflowGraph::mapFilters_ptr==0) CWorkflowGraph::mapFilters_ptr = new std::unordered_map <int, StdString>;

      (*CWorkflowGraph::mapFilters_ptr)[this->filterID] = "F "+op_expression +" S Filter" ;
      std::cout<<"CFieldScalarArithmeticFilter::apply filter tag = "<<this->tag<<std::endl;

      StdString str = data[0]->fieldID +" ts=" + to_string(data[0]->timestamp);
      (*CWorkflowGraph::mapFieldToFilters_ptr)[str].push_back(data[0]->src_filterID);
      (*CWorkflowGraph::mapFieldToFilters_ptr)[str].push_back(this->filterID);
    }

    CDataPacketPtr packet(new CDataPacket);
    packet->date = data[0]->date;
    packet->timestamp = data[0]->timestamp;
    packet->status = data[0]->status;
    if(this->tag) packet->src_filterID = this->filterID;
    packet->fieldID = this->output_field_id;
    

    if (packet->status == CDataPacket::NO_ERROR)
      packet->data.reference(op(data[0]->data, value));

    return packet;
  }

  CFieldFieldArithmeticFilter::CFieldFieldArithmeticFilter(CGarbageCollector& gc, const std::string& op)
    : CFilter(gc, 2, this)
    , op(operatorExpr.getOpFieldField(op))
  { 
    StdString input_op_expression=op;
    if(input_op_expression == "add")
      op_expression = "+";
    else if(input_op_expression == "minus")
      op_expression = "-";
    else if(input_op_expression == "mult")
      op_expression = "x";
    else if(input_op_expression == "div")
      op_expression = "/";
    else if(input_op_expression == "pow")
      op_expression = "^";
    else if(input_op_expression == "eq")
      op_expression = "=";
    else if(input_op_expression == "lt")
      op_expression = "<";
    else if(input_op_expression == "gt")
      op_expression = ">";
    else if(input_op_expression == "le")
      op_expression = "<=";
    else if(input_op_expression == "ge")
      op_expression = ">=";
    else if(input_op_expression == "ne")
      op_expression = "!=";
    else
      op_expression = " ";
  };

  CDataPacketPtr CFieldFieldArithmeticFilter::apply(std::vector<CDataPacketPtr> data)
  {
    if(this->tag)
    {

      this->filterID = InvalidableObject::filterIdGenerator++;
   
      if(CWorkflowGraph::mapFieldToFilters_ptr==0) CWorkflowGraph::mapFieldToFilters_ptr = new std::unordered_map <StdString, vector <int> >;
      if(CWorkflowGraph::mapFilters_ptr==0) CWorkflowGraph::mapFilters_ptr = new std::unordered_map <int, StdString>;

      (*CWorkflowGraph::mapFilters_ptr)[this->filterID] = "F "+op_expression +" F Filter" ;
      std::cout<<"CFieldFieldArithmeticFilter::apply filter tag = "<<this->tag<<std::endl;

      StdString str = data[0]->fieldID +" ts=" + to_string(data[0]->timestamp);
      (*CWorkflowGraph::mapFieldToFilters_ptr)[str].push_back(data[0]->src_filterID);
      (*CWorkflowGraph::mapFieldToFilters_ptr)[str].push_back(this->filterID);

      str = data[1]->fieldID +" ts=" + to_string(data[0]->timestamp);

      (*CWorkflowGraph::mapFieldToFilters_ptr)[str].push_back(data[1]->src_filterID);
      (*CWorkflowGraph::mapFieldToFilters_ptr)[str].push_back(this->filterID);
    }


    CDataPacketPtr packet(new CDataPacket);
    packet->date = data[0]->date;
    packet->timestamp = data[0]->timestamp;
    if(this->tag) packet->src_filterID = this->filterID;
    packet->fieldID = this->output_field_id;
    

    if (data[0]->status != CDataPacket::NO_ERROR)
      packet->status = data[0]->status;
    else if (data[1]->status != CDataPacket::NO_ERROR)
      packet->status = data[1]->status;
    else
    {
      packet->status = CDataPacket::NO_ERROR;
      packet->data.reference(op(data[0]->data, data[1]->data));
    }

    return packet;
  }

  StdString CScalarFieldArithmeticFilter::GetName(void)    { return StdString("CScalarFieldArithmeticFilter"); }
  StdString CFieldScalarArithmeticFilter::GetName(void)    { return StdString("CFieldScalarArithmeticFilter"); }
  StdString CFieldFieldArithmeticFilter::GetName(void)     { return StdString("CFieldFieldArithmeticFilter"); }


} // namespace xios
