#include "file_writer_filter.hpp"
#include "exception.hpp"
#include "field.hpp"
#include "utils.hpp"
#include "workflow_graph.hpp"
#include "graphviz.hpp"

using namespace ep_lib;

namespace xios
{
  CFileWriterFilter::CFileWriterFilter(CGarbageCollector& gc, CField* field, bool buildWorkflowGraph /* =false */)
    : CInputPin(gc, 1)
    , field(field)
  {
    if (!field)
      ERROR("CFileWriterFilter::CFileWriterFilter(CField* field)",
            "The field cannot be null.");
  }

  void CFileWriterFilter::onInputReady(std::vector<CDataPacketPtr> data)
  {
    if(this->tag)
    {
      this->filterID = InvalidableObject::filterIdGenerator++;

      if(CWorkflowGraph::mapFieldToFilters_ptr==0) CWorkflowGraph::mapFieldToFilters_ptr = new std::unordered_map <StdString, vector <int> >;
      if(CWorkflowGraph::mapFilters_ptr==0) CWorkflowGraph::mapFilters_ptr = new std::unordered_map <int, StdString>;

      (*CWorkflowGraph::mapFilters_ptr)[this->filterID] = "File Writer Filter";

      if(CXios::isClient) std::cout<<"CFileWriterFilter::apply filter tag = "<<this->tag<<std::endl;

      if(CXios::isClient) 
      {
        StdString str = this->output_field_id +" ts=" + to_string(data[0]->timestamp);
        // StdString str = data[0]->fieldID +" ts=" + to_string(data[0]->timestamp);
        (*CWorkflowGraph::mapFieldToFilters_ptr)[str].push_back(data[0]->src_filterID);
        (*CWorkflowGraph::mapFieldToFilters_ptr)[str].push_back(this->filterID);  
      }
    }
    
    const bool detectMissingValue = ( !field->default_value.isEmpty() &&
                               ( (!field->detect_missing_value.isEmpty() || field->detect_missing_value == true)
                                 || field->hasGridMask()) );

    CArray<double, 1> dataArray = (detectMissingValue) ? data[0]->data.copy() : data[0]->data;

    if (detectMissingValue)
    {
      const double missingValue = field->default_value;
      const size_t nbData = dataArray.numElements();
      for (size_t idx = 0; idx < nbData; ++idx)
      {
        if (NumTraits<double>::isNan(dataArray(idx)))
          dataArray(idx) = missingValue;
      }
    }

    field->sendUpdateData(dataArray);
    
    
    
  }

  bool CFileWriterFilter::mustAutoTrigger() const
  {
    return true;
  }

  bool CFileWriterFilter::isDataExpected(const CDate& date) const
  {
    return true;
  }



} // namespace xios
